//=============================================================================
// Plugin for RPG Maker MZ
// DrawCharAtMenu.js
//=============================================================================

// 2022.Mar.16 Ver1.0.0 First Release
// 2024.Mar.22 Ver1.1.0 Character in menu does Animation

/*:
 * @target MZ
 * @plugindesc [Ver1.1.0]At Menu Scene, Replace Face With Character Graphic
 * @author Sasuke KANNAZUKI
 *
 * @param doesAnimation
 * @text Does display walking anime
 * @desc If the actor cannot move or dead, it doesn't display walking anime.
 * @type boolean
 * @default true
 *
 * @param animeSpeed
 * @text Frequency
 * @desc Animation is fast if the number is small.
 * @type number
 * @min 1
 * @default 20
 *
 * @help This plugin does not provide plugin commands.
 * This plugin runs under RPG Maker MZ.
 * This plugin replaces face graphic with character's one.
 *
 * [Summary]
 * This plugin replaces all face graphics on menu such as status, equip,
 * and so on.
 * It doesn't replace face graphic in message window and so on.
 *
 * [License]
 * this plugin is released under MIT license.
 * http://opensource.org/licenses/mit-license.php
 */

/*:ja
 * @target MZ
 * @plugindesc [Ver1.1.0]メニューで顔グラフィックの代わりに歩行グラフィックを表示
 * @author 神無月サスケ
 *
 * @param doesAnimation
 * @text 歩行アニメ表示？
 * @desc trueにすると戦闘不能や行動不能のキャラ以外は歩行アニメを表示
 * @type boolean
 * @on 表示
 * @off 非表示
 * @default true
 *
 * @param animeSpeed
 * @text 歩行アニメ頻度
 * @desc フレーム数で設定。数を少なくするとアニメ頻度が上がる
 * @type number
 * @min 1
 * @default 20
 *
 * @help このプラグインには、プラグインコマンドはありません。
 * このプラグインは、RPGツクールMZに対応しています。
 * このプラグインは、メニューで顔グラフィックの代わりに歩行グラフィックを
 * 表示します。
 * メッセージウィンドウの顔グラフィックは置き換わりません。
 *
 * ■概要
 * メニュー画面の装備やスキルなど、全ての顔グラフィックが
 * 歩行グラフィックに置き換わります。
 *
 * ■ライセンス表記
 * このプラグインは MIT ライセンスで配布されます。
 * ご自由にお使いください。
 * http://opensource.org/licenses/mit-license.php
 */

(() => {
  const pluginName = 'DrawCharAtMenu';
  //
  // process parameters
  //
  const parameters = PluginManager.parameters(pluginName);
  const doesAnimation = eval(parameters['doesAnimation'] || "true");
  const animeSpeed = Number(parameters['animeSpeed'] || 20);

  //
  // initialization 
  //
  const _Window_StatusBase_initialize = Window_StatusBase.prototype.initialize;
  Window_StatusBase.prototype.initialize = function(rect) {
    _Window_StatusBase_initialize.call(this, rect);
    this._pattern = 1;
    this._stopCount = 0;
  };

  //
  // draw considering pattern
  //

  // !!overwrite!!
  Window_StatusBase.prototype.drawCharacter = function(characterName,
    characterIndex, x, y) {
    const bitmap = ImageManager.loadCharacter(characterName);
    const big = ImageManager.isBigCharacter(characterName);
    const pw = bitmap.width / (big ? 3 : 12);
    const ph = bitmap.height / (big ? 4 : 8);
    const n = big ? 0: characterIndex;
    const sx = ((n % 4) * 3 + this.pattern()) * pw; // 1 -> pattern
    const sy = Math.floor(n / 4) * 4 * ph;
    this.contents.blt(bitmap, sx, sy, pw, ph, x - pw / 2, y - ph);
  };

  //
  // update pattern
  //
  Window_StatusBase.prototype.updatePattern = function() {
    if (!doesAnimation) {
      this._pattern = 1;
    } else {
      this._pattern = (this._pattern + 1) % 4;
    }
  };

  Window_StatusBase.prototype.pattern = function() {
    return this._pattern < 3 ? this._pattern : 1;
  };

  Window_StatusBase.prototype.updateStop = function() {
    if (++this._stopCount > animeSpeed) {
      this.updatePattern();
      this.refresh();
      this._stopCount = 0;
    }
  };

  const _Window_StatusBase_update = Window_StatusBase.prototype.update;
  Window_StatusBase.prototype.update = function() {
    _Window_StatusBase_update.call(this);
    this.updateStop();
  };

  //
  // draw character graphic instead of face graphic
  //
  Window_StatusBase.prototype.drawActorFace = function(actor, x, y, w, h) {
    w = w || ImageManager.faceWidth;
    h = h || ImageManager.faceHeight;
    const x2 = Math.floor(x + w / 2);
    const y2 = y + Math.floor(h * 3 / 4);
    const pattern = this._pattern;
    if (actor.restriction() >= 4) {
      this._pattern = 1;
    }
    this.drawActorCharacter(actor, x2, y2);
    this._pattern = pattern;
  };
})();
